package str.bcel;

import  org.apache.bcel.Constants;
import  org.apache.bcel.classfile.*;
import  org.apache.bcel.generic.Type;
import  java.io.*;

public class EnclosingMethod extends Attribute
{
  public static void register()
  {
    Attribute.addAttributeReader("EnclosingMethod",
      new AttributeReader()
      {
	public Attribute createAttribute(
	  int name_index,
	  int length,
	  java.io.DataInputStream stream,
	  ConstantPool cp) 
	{
	  try
	  {
	    return new EnclosingMethod(name_index, length, stream, cp);
	  }
	  catch(Exception exc)
	  {
	    System.err.println("PANIC!");
	    exc.printStackTrace();
	    System.exit(1);
	  }

	  return null;
	}
      });
  }

  private int _class_index;
  private int _method_index;
  private ConstantPool _cp;
  private String _method_name;
  private String _signature;

  EnclosingMethod(
    int name_index,
    int length,
    DataInputStream stream,
    ConstantPool cp) throws IOException
  {
    this(name_index, length, stream.readUnsignedShort(), stream.readUnsignedShort(), cp);
  }

  public EnclosingMethod(
    int name_index, 
    int length, 
    int class_index,
    int method_index,
    ConstantPool cp)
  {
    super((byte)19, name_index, length, cp);
    _class_index = class_index;
    _method_index = method_index;
    _cp = cp;

    if(method_index != 0)
    {
      ConstantNameAndType c = (ConstantNameAndType) _cp.getConstant(method_index, Constants.CONSTANT_NameAndType);
      _method_name = c.getName(_cp);
      _signature = c.getSignature(_cp);
    }
  }

  public void accept(Visitor v)
  {
    System.err.println("Please don't visit me");
    System.exit(1);
  }

  public void dump(DataOutputStream file) throws IOException
  {
    super.dump(file);
    file.writeShort(_class_index);
    file.writeShort(_method_index);
  }    

  public int getClassIndex()
  {
    return _class_index;
  }    

  public String getClassName()
  {
    String result = _cp.getConstantString(_class_index, Constants.CONSTANT_Class);
    return Utility.compactClassName(result, false);
  }

  public int getMethodIndex()
  {
    return _method_index;
  }    

  public String getMethodName()
  {
    return _method_name;
  }

  public Type getReturnType()
  {
    return Type.getReturnType(_signature);
  }

  public Type[] getArgumentTypes()
  {
    return Type.getArgumentTypes(_signature);
  }

  public void setClassIndex(int class_index)
  {
    _class_index = class_index;
  }

  public void setMethodIndex(int method_index)
  {
    _method_index = method_index;
  }

  public String toString()
  {
    return "EnclosingMethod(...)";
  }

  public Attribute copy(ConstantPool cp)
  {
    System.err.println("Please don't copy me");
    System.exit(1);
    return null;
  }
}
