package str.classtree;

import static str.aterm.ATermUtil.*;
import java.util.*;
import aterm.*;

public enum Flag
{
  PUBLIC       (0x0001, "Public"),
  PRIVATE      (0x0002, "Private"),
  PROTECTED    (0x0004, "Protected"),
  STATIC       (0x0008, "Static"), 
  FINAL        (0x0010, "Final"),
  SYNCHRONIZED (0x0020, "Synchronized"),
  SUPER        (0x0020, "Super"),
  BRIDGE       (0x0040, "Bridge"),
  VOLATILE     (0x0040, "Volatile"),
  VARARGS      (0x0080, "Varargs"),
  TRANSIENT    (0x0080, "Transient"),
  NATIVE       (0x0100, "Native"),
  INTERFACE    (0x0200, "Interface"),
  ABSTRACT     (0x0400, "Abstract"),
  STRICT       (0x0800, "Strict"),
  SYNTHETIC    (0x1000, "Synthetic"),
  ANNOTATION   (0x2000, "Annotation"),
  ENUM         (0x4000, "Enum");

  private final int _val;
  private final ATerm _aterm;

  Flag(int val, String fun)
  {
    _val = val;
    _aterm = appl(fun);
  }

  public ATerm toATerm()
  {
    return _aterm;
  }

  public int getValue() 
  {
    return _val;
  }

  public boolean isSetInFlags(int flags)
  {
    return (flags & _val) != 0;
  }

  public static int createFlags(Set<Flag> flags)
  {
    int result = 0;

    for(Flag f : flags)
      result = result | f.getValue();

    return result;
  }

  public static Flag fromString(String flag)
  {
    return valueOf(flag.toUpperCase());
  }

  static
  {
    Set<Flag> classFlags = new HashSet<Flag>();
    classFlags.add(PUBLIC);
    classFlags.add(FINAL);
    classFlags.add(SUPER);
    classFlags.add(INTERFACE);
    classFlags.add(ABSTRACT);
    classFlags.add(SYNTHETIC);
    classFlags.add(ANNOTATION);
    classFlags.add(ENUM);
    FOR_CLASS = Collections.unmodifiableSet(classFlags);

    Set<Flag> methodFlags = new HashSet<Flag>();
    methodFlags.add(PUBLIC);
    methodFlags.add(PRIVATE);
    methodFlags.add(PROTECTED);
    methodFlags.add(STATIC);
    methodFlags.add(FINAL);
    methodFlags.add(SYNCHRONIZED);
    methodFlags.add(BRIDGE);
    methodFlags.add(VARARGS);
    methodFlags.add(NATIVE);
    methodFlags.add(ABSTRACT);
    methodFlags.add(STRICT);
    methodFlags.add(SYNTHETIC);
    FOR_METHOD = Collections.unmodifiableSet(methodFlags);

    Set<Flag> fieldFlags = new HashSet<Flag>();
    fieldFlags.add(PUBLIC);
    fieldFlags.add(PRIVATE);
    fieldFlags.add(PROTECTED);
    fieldFlags.add(STATIC);
    fieldFlags.add(FINAL);
    fieldFlags.add(VOLATILE);
    fieldFlags.add(TRANSIENT);
    fieldFlags.add(SYNTHETIC);
    fieldFlags.add(ENUM);
    FOR_FIELD = Collections.unmodifiableSet(fieldFlags);
  }

  public static final Set<Flag> FOR_CLASS;
  public static final Set<Flag> FOR_METHOD;
  public static final Set<Flag> FOR_FIELD;
}
