package str.classtree.atermify;

import org.apache.bcel.Constants;
import org.apache.bcel.classfile.*;
import aterm.ATerm;
import str.aterm.ATermifier;
import org.apache.bcel.generic.Type;
import static str.aterm.ATermUtil.*;

public abstract class AbstractConstantATermifier implements ATermifier<Constant>
{
  public abstract ConstantPool getConstantPool();

  private ATermifier<Type> _ftype;

  public AbstractConstantATermifier()
  {
    super();
    _ftype = TypeATermifier.SHARED;
  }

  public ATerm convert(Constant c)
  {
    ATerm result;

    switch(c.getTag())
    {
      case Constants.CONSTANT_Long :
	long val1 = ((ConstantLong)c).getBytes();
	result = appl("Long", astring(String.valueOf(val1)));
	break;
      case Constants.CONSTANT_Float :
	float val2 = ((ConstantFloat)c).getBytes();
	result = appl("Float", astring(Float.toString(val2)));
	break;
      case Constants.CONSTANT_Double :
	double val3 = ((ConstantDouble)c).getBytes();
	result = appl("Double", astring(Double.toString(val3)));
	break;
      case Constants.CONSTANT_Integer :
	int val4 = ((ConstantInteger)c).getBytes();
	result = appl("Int", astring(Integer.toString(val4)));
	break;
      case Constants.CONSTANT_String :
	String val5 = ((ConstantString)c).getBytes(getConstantPool());
	result = appl("String", astring(val5));
	break;
      case Constants.CONSTANT_Class :
	String val6 = ((ConstantClass)c).getBytes(getConstantPool());
        if (!val6.startsWith("[")) {
            val6 = "L" + val6 + ";";
        }

	result = appl("Class", _ftype.convert(Type.getType(val6)));
	break;
      default:
	throw new IllegalArgumentException("Not a valid constant: " + c.toString());
    }

    return result;
  }
}
