package str.classtree.atermify;

import org.apache.bcel.classfile.*;
import org.apache.bcel.generic.*;
import aterm.ATerm;
import aterm.ATermList;
import static str.classtree.atermify.ATermifyUtil.*;
import static str.aterm.ATermUtil.*;

public class CodeAttributeATermifier extends AbstractAttributeATermifier
{
  private MethodGen _mg;

  public CodeAttributeATermifier(MethodGen mg)
  {
    super();
    _mg = mg;
  }

  public ATerm convert(Attribute attr)
  {
    ATerm result;

    if(attr instanceof LineNumberTable)
      result = buildLineNumberTable();
    else if(attr instanceof LocalVariableTable)
      result = buildLocalVariableTable();
    else
      result = buildUnknownAttribute(attr);

    return result;
  }

  public ATerm buildLocalVariableTable()
  {
    LocalVariableGen[] vars = _mg.getLocalVariables();

    ATermList list = _factory.makeList();
    for (int i = vars.length - 1; i >= 0; i--)
    {
      LocalVariableGen var = vars[i];
      int start = getInstructionPosition(_mg.getInstructionList(), var.getStart());
      int end   = getInstructionPosition(_mg.getInstructionList(), var.getEnd());

      list = list.insert(
	appl("Entry",
	  appl("Range", aint(start), aint(end)),
	  appl("Name", astring(var.getName())),
	  buildFieldDescriptor(var.getType()),
 	  appl("Index", aint(var.getIndex()))
	  ));
    }

    return appl("LocalVariableTable", list);
  }

  public ATerm buildLineNumberTable()
  {
    LineNumberGen[] lines = _mg.getLineNumbers();

    ATermList list = _factory.makeList();
    for (int i = lines.length - 1; i >= 0; i--)
    {
      LineNumberGen line = lines[i];
      int pos = getInstructionPosition(_mg.getInstructionList(), line.getInstruction());
      int src = line.getSourceLine();

      list = list.insert(
	appl("Entry", appl("Instruction", aint(pos)), appl("Line", aint(src))));
    }

    return appl("LineNumberTable", list);
  }
}
